'+--------------------------------------------------------------------------+
'|                                                                          |
'|                               zXref 1.10                                 |
'|                                                                          |
'|               Based on PBcodec, Source Code Checker, v1.14               |
'|                           from Brje Hagsten                             |
'|                                                                          |
'+--------------------------------------------------------------------------+
'|                                                                          |
'|                 PB9+ full SDK version by Patrice TERRIER                 |
'|                            copyright(c) 2009                             |
'|                           www.zapsolution.com                            |
'|                        pterrier@zapsolution.com                          |
'|                                                                          |
'+--------------------------------------------------------------------------+
'|                  Project started on : 08-12-2009 (MM-DD-YYYY)            |
'|                        Last revised : 09-13-2009 (MM-DD-YYYY)            |
'+--------------------------------------------------------------------------+
'
'  Scans a main source file and all its include files for Un-used Subs,
'  Functions, and variables. Version 1.1 also has ability to extract string
'  literals, that is, strings withing double-quotes.
'
'  PBcodec will save the result in a file ending with --LOG.TXT and also show
'  this file in associated texteditor.
'
'  This version does not process any Conditional Compiling statements, so far,
'  but all files are properly scanned and it even handles "DIM x AS GLOBAL in
'  a correct way. Tested on my own projects, all reports so far has been correct.
'
'  The output will show the SUB,FUNCT and VAR name followed by  [FileName: 565
'  where the last number is the line number where the item is declared. It will
'  also present a list of all Includes, Global vars, Subs and Functions, this
'  part originally written by Wayne Diamond.
'
'  Public Domain, this version by Borje Hagsten, July 2001, but main credits
'  goes to Scott Slater for showing us it could be done (and how to do it).
'  Parts of this program has been copied from his version, but parsing and
'  some other things has been totally rewritten, my way..
'  Many have been involved, giving many valuable tips and providing code,
'  so one could say this is the result of a joined PB community effort.. :)
'
'  Tip: Prog launches txt-associated texteditor to show report. Standard is
'  NotePad. I have set mine to use Courier New, 9p, which gives nice output..
'
'  A few notes: Exported subs/functions, or ones inside "public" include files,
'  may have a reference count of zero and still not be un-used, since some other
'  program may be using them.
'
'  Also, since one of the advantages with the PB language is almost unlimited
'  flexibility to do things, there's no guaranties everything will be found,
'  or even be 100% correct. It has proved to be successful on my projects,
'  but please, use this (and similar) programs with extreme care..
'
' History:
'  Sep 13, 2009 - ISFILE has been replaced by zExist.
'  Aug 31, 2009 - Cleaning of the code + Progressbar.
'  Aug 30, 2009 - Parsing #IF in include file to detect un-used sections.
'  Aug 18, 2009 - Inclean code has been added to the project.
'  Aug 16, 2009 - Added config file to customize the scan path, and to support
'          Jos Roca's include file.
'  Aug 12, 2009 - The DDT syntax has been replaced by plain SDK coding style,
'          un-needed API encapsulation have been removed to speed up code,
'          and reduce the size of the final exe.  
'  Aug 11, 2003 - added some extrs checks for line wraps, _ to DoProcess
'  Jan 23, 2002 - added support for relative include paths, plus optimized
'          some for better speed, etc.
'  Jan 23, 2002 - added support for relative include paths, plus optimized
'          some for better speed, etc.
'  Jan 17, 2002 - changed to use IsCharAlphaNumeric in ANSItrim, to include
'          leading/trailing digits in string literals.
'  Oct 17, Corrected error in DoSaveResults, where global/local name mix warning
'          could end up pointing at wrong file for first local declare.
'  Oct 10, added exclude() array to avoid some of the most common include file
'          names when extracting string literals. Possible to expand - see WinMain.
'          Also set string literal extraction checkbox as checked from start.
'  Oct 09, added possibility to extract string literals, that is, text within
'          double-quotes. Added AnsiTRIM function for this purpose.
'  Aug 01, in DoSaveResults, moved REDIM PRESERVE gaUnusedLocal out from loop in first
'          IF/THEN block and reversed loop, because it sometimes GPF'd there.
'          Of course it could GPF. Must run such loops backwards, silly me.
'  Aug 01, removed AllLocals array and use gaUnusedLocal to store all locals instead.
'          Changed report accordingly and now, Global/Local name mix lists
'          Line number for local representations too.
'  Jul 31, excluded declarations of Subs/Function from usage check
'  Jul 31, re-fixed previous stupid fix of GLOBAL DIM, so it works this time..
'  Jul 29, fixed code in DoProcess - ExtractLocals, to check DIM more carefully,
'          since DIM/REDIM may have been preceeded with a GLOBAL declare of same variable.
'  Jul 29, added support for multiple include file paths in WinMain and DoGetIncFiles
'          Added check for trailing backslash to fExist, so paths are handled correctly
'  Jul 29, added code to DoProcess - ExtractSub, to exclude declares for external
'          procedures (in DLL's etc.) from being counted as "declared but un-used".
'          Also added code to DoProcess - ExtractLine, to replace colons within
'          paranthesis, which could cause weird results when parsing a line for colons.
'  Jul 29, added "Scan" button to enable easy rescanning of a file, since I
'          have found this useful to do after changes have been made. Also did
'          some minor tweaking of the code to enhance performance.
'  Jul 28, major trimming of parser, to ensure results and improve performance.

#COMPILE EXE "zXref.exe"
#OPTION VERSION5

'#OPTIMIZE SIZE
'#OPTIMIZE SPEED

#RESOURCE "zXref.pbr"

#INCLUDE "zXref.inc"
'----------------------------------------------------------------------

%EXTEND_EXTRA       = 10
%IDC_HELPER         = 114
%IDC_PROGRESS       = 115
%IDC_MAINFILE       = 116
%IDC_BROWSE         = 120
%IDC_CHECKBOX       = 130
%IDC_CHKBOXTEXT     = 140
%IDC_CHKBOXCLEAN    = 150
%IDC_CHKBOXCLEANTXT = 160

%CLIENTWIDTH        = 248
%CLIENTHEIGHT       = 140

$VERSION            = "1.10"
'----------------------------------------------------------------------

TYPE InfoStruct
    uName       AS ASCIIZ * 100 'is 100 enough? For me it is, but if not for you, increase..
    zName       AS ASCIIZ * 100
    inFunct     AS ASCIIZ * 100
    IsUsed      AS LONG
    iType       AS LONG
    FileNum     AS LONG
    LineNum     AS LONG
    SubEnd      AS LONG
    Exported    AS LONG
END TYPE

GLOBAL gaLocal()            AS InfoStruct  ' All Locals in Current Proc (TEMP)
GLOBAL gaGlobal()           AS InfoStruct  ' All Globals
GLOBAL gaDuplicateGlobal()  AS InfoStruct  ' Duplicate Global names
GLOBAL gaDeclareSubFunc()   AS InfoStruct  ' Declared Subs/Functions
GLOBAL gaUnusedLocal()      AS InfoStruct  ' Un-Used Locals
GLOBAL gaFunctions()        AS InfoStruct  ' All Functions
GLOBAL gsFiles()            AS STRING
GLOBAL gsIncludes()         AS STRING      ' Include files
GLOBAL gsString()           AS STRING      ' For string literals
GLOBAL gnStrCount           AS LONG

GLOBAL gnLineCount          AS LONG
GLOBAL gnIndent             AS LONG

GLOBAL gnCountIF            AS LONG        ' To perform #IF detection
GLOBAL gnFlagDEF            AS LONG        ' To Perform %DEF detection in #IF

GLOBAL gsFilePath, gsFileName, gsDestFile AS STRING

GLOBAL gnDuplicateGlobal AS LONG       ' # of Duplicate Globals
GLOBAL gnLocal           AS LONG       ' # of gaLocal
GLOBAL gnUnusedLocal     AS LONG       ' # of gaUnusedLocal
GLOBAL gnGlobal          AS LONG       ' # of gaGlobal
GLOBAL gnFunctions       AS LONG       ' # of gaFunctions
GLOBAL gnDeclCount       AS LONG
GLOBAL gnTotLines        AS LONG
GLOBAL gdwTick           AS DWORD      ' Elapsed time

DECLARE FUNCTION AnsiTRIM(BYVAL txt AS STRING) AS STRING
DECLARE SUB DoGetIncFiles(BYVAL sFileName AS STRING)
DECLARE FUNCTION DoProcess(BYVAL hWnd AS LONG, BYVAL sFileName AS STRING, BYVAL fNum AS LONG, BYVAL WhatRun AS LONG) AS LONG
DECLARE FUNCTION GetDroppedFile(BYVAL hDrop AS LONG, Fi() AS STRING) AS LONG
DECLARE FUNCTION IsFileMain(BYVAL fName AS STRING) AS LONG
DECLARE SUB DoInitProcess(BYVAL hWnd AS LONG, BYVAL fName AS STRING)
DECLARE SUB DoSaveResults
DECLARE SUB UCASES(a$)

'// Main program entrance
FUNCTION WINMAIN (BYVAL hCurInstance  AS LONG, _
                  BYVAL hPrevInstance AS LONG, _
                  BYVAL lpszCmdLine   AS ASCIIZ PTR, _
                  BYVAL nCmdShow      AS LONG) AS LONG

    LOCAL zClass               AS ASCIIZ * 80
    LOCAL hWinXP_Lib           AS DWORD ' Handle to WinXP Theme DLL
    LOCAL hWinXP_IsThemeActive AS DWORD ' Handle to WinXP's IsThemeActive function
    LOCAL IsInitialized        AS LONG
    LOCAL dwExStyle            AS DWORD
    LOCAL dwStyle              AS DWORD
    LOCAL hInstance            AS DWORD
    LOCAL Msg                  AS tagMsg
    LOCAL wc                   AS WndClassEx
    LOCAL hWnd, hCtrl, hFont   AS DWORD
    LOCAL rw, rc               AS RECT

    hInstance = GetModuleHandle("")

    hFont = GetStockObject(%ANSI_VAR_FONT)

    zClass = "zXref"
    wc.cbSize = SIZEOF(wc)
    IsInitialized = GetClassInfoEx(hInstance, zClass, wc)
    IF IsInitialized    = 0 THEN
       wc.cbSize        = SIZEOF(wc)
       wc.style         = 0'//%CS_HREDRAW OR %CS_VREDRAW OR %CS_DBLCLKS
       wc.lpfnWndProc   = CODEPTR(WndProc)
       wc.cbClsExtra    = 0
       wc.cbWndExtra    = %EXTEND_EXTRA * 4
       wc.hInstance     = hInstance
       wc.hIcon         = LoadIcon(wc.hInstance, "PROGRAM")
       wc.hCursor       = LoadCursor(%NULL, BYVAL %IDC_ARROW)
       wc.hbrBackground = GetStockObject(0)
       wc.lpszMenuName  = %NULL
       wc.lpszClassName = VARPTR(zClass)
       wc.hIconSm       = wc.hIcon
       IF RegisterClassEx(wc) THEN IsInitialized = %TRUE
    END IF
'
    IF IsInitialized THEN

     ' Load the WinXP Theme support (if applicable)
       hWinXP_Lib = LoadLibrary("UxTheme.dll")
       IF hWinXP_Lib THEN
          hWinXP_IsThemeActive = GetProcAddress(hWinXP_Lib, "IsThemeActive")
       END IF

     ' Window Extended Style
       dwExStyle = %WS_EX_ACCEPTFILES OR %WS_EX_TOPMOST OR %WS_EX_CONTROLPARENT OR %WS_EX_WINDOWEDGE
     ' Windows Style
       dwStyle = %WS_POPUP OR %WS_CAPTION OR %WS_SYSMENU OR %WS_MINIMIZEBOX OR %WS_VISIBLE

       CALL SetRect(rc, 0, 0, %CLIENTWIDTH, %CLIENTHEIGHT)
'      // Note: indeed we don't need AdjustWindowRectEx, because we use do not use a non-client area
'      // but it won't hurt anything to keep it, in case we change our mind ;)
       CALL AdjustWindowRectEx(rc, dwStyle, %FALSE, dwExStyle)  ' Adjust Window To True Requested Size
       rc.nRight = rc.nRight - rc.nLeft: rc.nBottom = rc.nBottom - rc.nTop
       CALL SystemParametersInfo(%SPI_GETWORKAREA, BYVAL 0, BYVAL VARPTR(rw), 0) ' Main monitor coordinates
       hWnd = CreateWindowEx(dwExStyle, zClass, zClass + $SPC + $VERSION, dwStyle, _
                              rw.nRight - (rc.nRight + 4), rw.nBottom - (rc.nBottom + 5), _
                              rc.nRight, _                       ' Calculate Window Width
                              rc.nBottom, _                      ' Calculate Window Height
                              %NULL, _                           ' No Parent Window
                              %NULL, _                           ' No Menu
                              wc.hInstance, _                    ' Instance
                              BYVAL %NULL)                       ' Dont Pass Anything To WM_CREATE

       IF hWnd THEN
          CALL zMainWindow(hWnd)

          hCtrl = CreateWindowEx(%WS_EX_CLIENTEDGE, "Static", _
                                 "Drag && Drop a main source file on dialog," + $CRLF + "or use Browse to select a file to Scan..", _
                                 %WS_CHILD OR %WS_VISIBLE OR %SS_CENTER, _
                                 3, 3, 242, 33, hWnd, %IDC_HELPER, hInstance, BYVAL %NULL)
                                 IF hFont THEN CALL SendMessage(hCtrl, %WM_SETFONT, hFont, 0)

          hCtrl = CreateWindowEx(%WS_EX_CLIENTEDGE, "msctls_progress32", _
                                 "", %WS_CHILD OR %WS_VISIBLE, _
                                 3, 38, 242, 20, hWnd, %IDC_PROGRESS, hInstance, BYVAL %NULL)       
'         // Set the range
          CALL SendMessage(hCtrl, 1030, 0, 100)
'         // Set initial value to null
          CALL SendMessage(hCtrl, 1026, 0, 0)
'         // Set the color
          CALL SendMessage(hCtrl, 1033, 0, RGB(0, 255, 0))
         
          hCtrl = CreateWindowEx(0, "Static", _
                                 " Main file: ", _
                                 %WS_CHILD OR %WS_VISIBLE, _
                                 3, 44 + 24, 242, 16, hWnd, %IDC_MAINFILE, hInstance, BYVAL %NULL)
                                 IF hFont THEN CALL SendMessage(hCtrl, %WM_SETFONT, hFont, 0)
         
          hCtrl = CreateWindowEx(0, "Button", _
                                 " Browse...", _
                                 %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP, _
                                 6, 86 + 24, 75, 23, hWnd, %IDC_BROWSE, hInstance, BYVAL %NULL)
                                 IF hFont THEN CALL SendMessage(hCtrl, %WM_SETFONT, hFont, 0)
         
          hCtrl = CreateWindowEx(0, "Button", _
                                 "Scan", _
                                 %WS_CHILD OR %WS_VISIBLE OR %WS_DISABLED OR %WS_TABSTOP, _
                                 87, 86 + 24, 75, 23, hWnd, %IDOK, hInstance, BYVAL %NULL)
                                 IF hFont THEN CALL SendMessage(hCtrl, %WM_SETFONT, hFont, 0)
         
          hCtrl = CreateWindowEx(0, "Button", _
                                 "Quit", _
                                 %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP, _
                                 168, 86 + 24, 75, 23, hWnd, %IDCANCEL, hInstance, BYVAL %NULL)
                                 IF hFont THEN CALL SendMessage(hCtrl, %WM_SETFONT, hFont, 0)
         
          hCtrl = CreateWindowEx(0, "Button", _
                                 "", _
                                 %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR %BS_AUTOCHECKBOX, _
                                 6, 63 + 24, 13, 13, hWnd, %IDC_CHECKBOX, hInstance, BYVAL %NULL)
                                 IF hFont THEN CALL SendMessage(hCtrl, %WM_SETFONT, hFont, 0)

          hCtrl = CreateWindowEx(0, "Static", "Extract string literals", _
                                 %WS_CHILD OR %WS_VISIBLE, _
                                 23, 63 + 24, 120, 16, hWnd, %IDC_CHKBOXTEXT, hInstance, BYVAL %NULL)
                                 IF hFont THEN CALL SendMessage(hCtrl, %WM_SETFONT, hFont, 0)
         
          hCtrl = CreateWindowEx(0, "Button", _
                                 "", _
                                 %WS_CHILD OR %WS_VISIBLE OR %WS_TABSTOP OR %BS_AUTOCHECKBOX, _
                                 169, 63 + 24, 13, 13, hWnd, %IDC_CHKBOXCLEAN, hInstance, BYVAL %NULL)
                                 IF hFont THEN CALL SendMessage(hCtrl, %WM_SETFONT, hFont, 0)

          hCtrl = CreateWindowEx(0, "Static", "Clean code", _
                                 %WS_CHILD OR %WS_VISIBLE, _
                                 186, 63 + 24, 90, 16, hWnd, %IDC_CHKBOXCLEANTXT, hInstance, BYVAL %NULL)
                                 IF hFont THEN CALL SendMessage(hCtrl, %WM_SETFONT, hFont, 0)

          'CALL SendMessage(zGetMainItem(%IDC_CHECKBOX), %BM_SETCHECK, %BST_CHECKED, 0)


          CALL DragAcceptFiles(hWnd, %TRUE) 'enable drag&drop

'         // Retrieve the zXref config file setup
          CALL ReadConfig()

          CALL ShowWindow(hWnd, nCmdShow)
          CALL UpdateWindow(hWnd)
         
          WHILE GetMessage(Msg, %NULL, 0, 0)
             TranslateMessage Msg
             DispatchMessage Msg
          WEND

          FUNCTION = msg.wParam
       END IF

'      // UNLOAD the WinXP Theme DLL (if necessary)
       IF hWinXP_Lib THEN CALL FreeLibrary(hWinXP_Lib)

    END IF
END FUNCTION

'// Window procedure
FUNCTION WndProc (BYVAL hWnd AS DWORD, BYVAL Msg AS LONG, BYVAL wParam AS LONG, BYVAL lParam AS LONG) AS LONG

    SELECT CASE LONG Msg
    CASE %WM_CREATE
         'A good place to initiate things, declare variables,
         'create controls and read/set settings from a file, etc.
         '-------------------------------------------------------

    CASE %WM_COMMAND
         'Messages from controls and menu items are handled here.
         '-------------------------------------------------------
         SELECT CASE LONG LOWRD(wParam)
         CASE %IDCANCEL
              IF HIWRD(wParam) = %BN_CLICKED THEN 
                 CALL SendMessage(hWnd, %WM_DESTROY, wParam, lParam)
                 FUNCTION = 0 : EXIT FUNCTION
              END IF

         CASE %IDC_BROWSE
              LOCAL dwStyle AS DWORD, fName AS STRING, Buffer AS STRING, Path AS STRING

              dwStyle = %OFN_EXPLORER OR %OFN_FILEMUSTEXIST OR %OFN_HIDEREADONLY
              fName   = "*.BAS"
              Buffer  = "PB Code files (*.BAS)|*.BAS|"
              Path    = CURDIR$

              IF OpenFileDialog(hWnd, "", fName, Path, Buffer, "BAS" , dwStyle) THEN
                 REDIM gsFiles(1)  ' Reset Array
                 gsFiles(0) = fName
                 fName = PATHNAME$(NAMEX, fName)
                 CALL zSetCTLText(zGetMainItem(%IDC_MAINFILE), " Main file: " + UCASE$(fName))
                 CALL EnableWindow(zGetMainItem(%IDOK), %TRUE)
                 CALL DoInitProcess(hWnd, gsFiles(0)) '<- deactivate, if not to scan directly..
              END IF
              FUNCTION = 0 : EXIT FUNCTION

         CASE %IDOK
              REDIM PRESERVE gsFiles(1)
              CALL DoInitProcess(hWnd, gsFiles(0)) 'scan file
              FUNCTION = 0 : EXIT FUNCTION

         END SELECT

    CASE %WM_CTLCOLORSTATIC
         IF lParam = zGetMainItem(%IDC_HELPER) THEN
            SetBkColor wParam, GetSysColor(%COLOR_INFOBK)
            FUNCTION = GetSysColorBrush(%COLOR_INFOBK)
         ELSE
            SetBkColor wParam, GetSysColor(%COLOR_WINDOW)
            FUNCTION = GetStockObject(%NULL_BRUSH)
         END IF
         EXIT FUNCTION

    CASE %WM_DROPFILES
         REDIM gsFiles(1) ' Reset Array
         IF GetDroppedFile(wParam, gsFiles()) THEN       ' Retrieve the Dropped filenames
            CALL DoInitProcess(hWnd, gsFiles(0))
         END IF

    CASE %WM_DESTROY
         CALL DragAcceptFiles(hWnd, 0)
         ' is sent when program ends - a good place to delete any created objects and
         ' store settings in file for next run, etc. Must send PostQuitMessage to end
         ' properly in SDK-style dialogs. The PostQuitMessage function sends a WM_QUIT
         ' message to the program's (thread's) message queue, and then WM_QUIT causes
         ' the GetMessage function to return zero in WINMAIN's message loop.
         '----------------------------------------------------------------------------
         PostQuitMessage 0
         FUNCTION = 0 : EXIT FUNCTION

    END SELECT

    FUNCTION = DefWindowProc(hWnd, Msg, wParam, lParam)
END FUNCTION

FUNCTION GetCheckButtonStatus(BYVAL hWnd AS LONG) AS LONG
    IF SendMessage(hWnd, %BM_GETCHECK, 0, 0) = %BST_CHECKED THEN
       FUNCTION = -1
    END IF
END FUNCTION

'// #STACK 8388608 ' (8Mb)

'// Initiate and run entire process
SUB DoInitProcess(BYVAL hWnd AS LONG, BYVAL sFileName AS STRING)
    LOCAL ci AS LONG, mc AS LONG, sTmp AS STRING
'// LOCAL cs AS CRITICAL_SECTION
    
    IF zExist(sFileName) THEN 'make sure it exists
       ci = IsFileMain(sFileName)  'if a #COMPILE statement exists (main source file)
  
       IF ci = -3 THEN         'if return is -3, file was empty
          MSGBOX "Selected file is empty.",,"Error!"
          EXIT SUB
  
       ELSEIF ci = -2 THEN     'if return is -2, file could not be opened
          MSGBOX "Selected file could not be opened!",,"Error!"
          EXIT SUB
  
       ELSEIF ci >= 0 THEN     'if return is >= 0, file was not a main source file
          MSGBOX "Selected file does not contain a #COMPILE statement.",,"Error!"
          EXIT SUB
       END IF
  
    ELSE                       'else, it didn't even exist..
       MSGBOX "Could not open this file:" + $CRLF + sFileName + $CRLF + $CRLF + _
              "Please make sure it exists and try again.",,"Error!"
       EXIT SUB
    END IF

    gsFilePath = PATHNAME$(PATH, sFileName)
    gsFileName = PATHNAME$(NAMEX, sFileName)
    CHDRIVE LEFT$(gsFilePath, 2)
    CHDIR gsFilePath

'   /////////////////////////////////////////////////
'   // Patrice
'   // Build "combined source code" using recursivity
    IF GetCheckButtonStatus(zGetMainItem(%IDC_CHKBOXCLEAN)) THEN

    REDIM gsIncludes(0) ' Version 1.05
    ListResetAll() '// Reste all hidden listbox
    CALL zSetCTLText(zGetMainItem(%IDC_HELPER), "Creating single combined source code file")
    gsFileName = "@" + gsFileName
    gnLineCount = 0: gnIndent = 0: gnCountIF = 0: gnFlagDEF = 0
'   // Reset ProgressBar to null
    CALL SendMessage(zGetMainItem(%IDC_PROGRESS), 1026, 0, 0)
    CALL zUseBuildName(gsFilePath + gsFileName, 1)
    CALL BuildSingleFile(gsFiles(0))
    CALL zBuild("", 0) ' Close Build file
'   // Use merged files instead of the initial file name. 
    gsFiles(0) = "": sFileName = gsFilePath + gsFileName

    END IF
'   /////////////////////////////////////////////////
     
    CALL EnableWindow(zGetMainItem(%IDC_BROWSE), %FALSE)
    CALL EnableWindow(zGetMainItem(%IDOK), %FALSE)
    CALL EnableWindow(zGetMainItem(%IDC_CHECKBOX), %FALSE)
  
    CALL zSetCTLText(zGetMainItem(%IDC_MAINFILE), " Main file: " + UCASE$(gsFileName))
  
    REDIM gsIncludes(0)
    IF gsFiles(0) = "" THEN gsFiles(0) = sFileName
 
'// CALL InitializeCriticalSection(cs)
'// CALL EnterCriticalSection(cs)

    CALL zSetCTLText(zGetMainItem(%IDC_HELPER), "Collecting include files")
    CALL DoGetIncFiles(sFileName)
  
    gnFunctions = 0:       REDIM gaFunctions(0)  
    gnGlobal  = 0:         REDIM gaGlobal(0)
    gnUnusedLocal  = 0:    REDIM gaUnusedLocal(0)
    gnLocal   = 0:         REDIM gaLocal(0)
    gnDuplicateGlobal = 0: REDIM gaDuplicateGlobal(0)
    gnDeclCount = 0:       REDIM gaDeclareSubFunc(0)
    gnStrCount = 0:        REDIM gsString(0)
    gnTotLines = 0
    gdwTick = GetTickCount()
  
    FOR mc = 0 TO 1
        FOR ci = 0 TO UBOUND(gsFiles) - 1
            sTmp = PATHNAME$(NAMEX, gsFiles(ci))
            IF CheckInclude(sTmp) = 0 THEN
               IF mc = 0 THEN
                  CALL zSetCTLText(zGetMainItem(%IDC_HELPER), "Scanning for Local vars, Subs and Functions in: " + sTmp)
               ELSE
                  CALL zSetCTLText(zGetMainItem(%IDC_HELPER), "Scanning for Global vars in: " + sTmp)
               END IF
               CALL DoProcess(hWnd, gsFiles(ci), ci, mc)
            END IF
        NEXT
        IF mc = 0 THEN
           IF gnFunctions       THEN REDIM PRESERVE gaFunctions(gnFunctions)
           IF gnGlobal          THEN REDIM PRESERVE gaGlobal(gnGlobal)
           IF gnUnusedLocal     THEN REDIM PRESERVE gaUnusedLocal(gnUnusedLocal)
           IF gnDuplicateGlobal THEN REDIM PRESERVE gaDuplicateGlobal(gnDuplicateGlobal)
           IF gnDeclCount       THEN REDIM PRESERVE gaDeclareSubFunc(gnDeclCount)
           IF gnStrCount        THEN REDIM PRESERVE gsString(gnStrCount)
        END IF
    NEXT
    gdwTick = GetTickCount() - gdwTick

'// CALL LeaveCriticalSection(cs)
'// CALL DeleteCriticalSection(cs)

    CALL EnableWindow(zGetMainItem(%IDC_BROWSE), %TRUE)
    CALL EnableWindow(zGetMainItem(%IDOK), %TRUE)
    CALL EnableWindow(zGetMainItem(%IDC_CHECKBOX), %TRUE)
    CALL zSetCTLText(zGetMainItem(%IDC_HELPER), "Done! Drag && Drop a main source file on dialog, " + $CRLF + _
                                "or use Browse to select a file to Scan.")
    CALL DoSaveResults

END SUB

'// Get all included files into array
SUB DoGetIncFiles(BYVAL sFileName AS STRING)
    LOCAL K, One, hFile, ci, p, fc, nFound, nASC AS LONG
    LOCAL pbLetter AS BYTE PTR, sWork, sTemp, sTemp2, sLineStr AS STRING

'   // Include only once!
    sTemp = UCASE$(PATHNAME$(NAMEX, sFileName))
'   //Wrong syntax with standard string array
'   // ARRAY SCAN gsIncludes(), FROM 1 TO LEN(sTemp), COLLATE UCASE, = sTemp, TO nFound
    ARRAY SCAN gsIncludes(), COLLATE UCASE, = sTemp, TO nFound
    IF nFound THEN EXIT SUB

    IF zExist(sFileName) THEN
       IF zFOpen((sFileName), 0, 2, hFile) = 0 THEN
          sTemp = SPACE$(zFLof(hFile))
          CALL zFGet(hFile, sTemp)
          CALL zFClose(hFile)
       END IF
    END IF
    IF LEN(sTemp) < 2 THEN
       MSGBOX sFileName + $cr + $cr + "Selected file could not be found!",,"Error!"
       EXIT SUB
    END IF

    REDIM tmpFiles(0) AS STRING
    p = 1: ci = 0
    pbLetter = STRPTR(sTemp)
  
    FOR ci = 1 TO LEN(sTemp)
        IF @pbLetter = 13 THEN
           IF ci - p > 2 THEN                             'if more than a line feed
              sLineStr = UCASE$(LTRIM$(MID$(sTemp, p, ci - p)))
              nASC = ASC(sLineStr): IF nASC = 36 THEN nASC = 35
              IF nASC = 35 AND MID$(sLineStr, 2, 8) = "INCLUDE " THEN       'can be several INCLUDE statements in file, anywhere in code..
                 sWork = PARSE$(sLineStr, CHR$(34), 2)     'get filename
'                // Patrice
'                // Check if it starts with a dot ".\", "..\", "...\", "....\", ".....\", "......\"
                 IF ASC(sWork) = 46 THEN 
                    sTemp2 = gsFilePath
                    K = 1: One = 0
                    WHILE ASC(sWork, K) = 46
                        K += 1
                        IF ASC(sWork, K) = 92 THEN
                           sWork = sTemp2 + MID$(sWork, K + One)
                        ELSE
                           sTemp2 = SplitPath(sTemp2): One = 1
                        END IF
                    WEND
                 END IF
       
                 IF INSTR(-1, sWork, ".") = 0 THEN           'if no file extension is given,
                    sWork = sWork + ".BAS"                   'compiler assumes .BAS file
                 END IF
       
                 IF LEN(gsFilePath) AND zExist(gsFilePath + sWork) THEN 'try with current file's path
                    sWork = gsFilePath + sWork              'if ok, use it
                 END IF
       
'                // Patrice resolve missing include path 
                 IF zExist(sWork) = 0 THEN sWork = zFindFile(sWork)
                 IF zExist(sWork) THEN                       'safety check - if we can find what we got..
                    tmpFiles(fc) = sWork                     'store path + name in temporary array
                    fc += 1: REDIM PRESERVE tmpFiles(fc)     'incr counter and redim temporary array
                 END IF
              END IF
           END IF
           pbLetter += 1: ci += 1: p = ci + 1                'jump past LF and store position
        END IF
        pbLetter += 1
    NEXT
  
    IF fc THEN
       p = UBOUND(gsFiles)
       REDIM PRESERVE gsFiles(p + fc)
       REDIM PRESERVE gsIncludes(p + fc)
  
       FOR ci = 0 TO fc - 1
          gsFiles(p + ci)  = tmpFiles(ci)
          gsIncludes(p + ci) = PATHNAME$(NAMEX, sFileName)
       NEXT
       FOR ci = 0 TO fc - 1
           IF CheckInclude(PATHNAME$(NAMEX, tmpFiles(ci))) = 0 THEN
              CALL DoGetIncFiles(tmpFiles(ci)) 'recursive call to get eventual includes in includes
           END IF
       NEXT
    END IF
  
END SUB

'// Trim away all leading/ending non-letters and digits from a string
FUNCTION AnsiTRIM(BYVAL txt AS STRING) AS STRING
    LOCAL pos1 AS LONG, pos2 AS LONG
    
    FOR pos1 = 1 TO LEN(txt)
       IF IsCharAlphaNumeric(ASC(txt, pos1)) THEN EXIT FOR
    NEXT
    FOR pos2 = LEN(txt) TO 1 STEP -1
       IF IsCharAlphaNumeric(ASC(txt, pos2)) THEN EXIT FOR
    NEXT
    
    IF Pos2 > Pos1 THEN FUNCTION = MID$(txt, Pos1, Pos2 - Pos1 + 1)

END FUNCTION

'// Load text from file, extract lines and get all subs, functions and globals into arrays.
FUNCTION DoProcess(BYVAL hWnd AS LONG, BYVAL sFileName AS STRING, BYVAL fNum AS LONG, BYVAL WhatRun AS LONG) AS LONG
    LOCAL bpLetter, bpLetter2 AS BYTE PTR
    LOCAL I, hFile, nCi, nP, nAc, K, nExFlag, nQuotePos, nQuotePos2, nDup, ndUscored AS LONG
    LOCAL nExported, nX, nY, nUscoredGlobal, nEndRout, nUscored AS LONG
    LOCAL nDi, nPos, nWordFlag, nStrFlag, nFlag, nPar AS LONG
    LOCAL nLen, nLocX, nLocY, nLocPos, nUscoredLocal, nIv, nIsGlobal AS LONG
    LOCAL sWork, sMainStr, sBuf, sTxt, sFname, sDump AS STRING
    LOCAL nGetStrings AS LONG

'   // Load file into sMainStr
    nLen = 0
    IF zExist(sFileName) THEN  ' If file doesn't exist then exit!
       IF zFOpen((sFileName), 0, 2, hFile) = 0 THEN
          nLen = zFLof(hFile)
          sMainStr = SPACE$(nLen)
          CALL zFGet(hFile, sMainStr)
          CALL zFClose(hFile)
       END IF
    END IF
    IF nLen < 2 THEN EXIT FUNCTION                   'if sMainStr is too short, exit

'   // Scan sMainStr and extract lines
    nP = 1: nCi = 0 : I = 0
    bpLetter = STRPTR(sMainStr)                      'point to beginning of sMainStr
    
    FOR nCi = 1 TO nLen
       SELECT CASE LONG @bpLetter
          CASE 9  : @bpLetter = 32                   'replace ev. tab with space
          CASE 13
             I += 1: gnTotLines += 1                 'increase line- and total line counter
             IF nCi - nP > 3 THEN GOSUB ExtractLine  'if line is enough long, extract it
             bpLetter += 1: nCi += 1: nP = nCi + 1   'jump past LF and store position
       END SELECT
       bpLetter += 1
    NEXT
    
    IF nCi > nP + 2 THEN
       I += 1                    'increase line counter
       GOSUB ExtractLine         'if line didn't end with line feed..
    END IF
    CALL SendMessage(zGetMainItem(%IDC_PROGRESS), 1026, 0, 0)

    EXIT FUNCTION

ExtractLine:
    CALL SendMessage(zGetMainItem(%IDC_PROGRESS), 1026, (nCi * 100) \ nLen, 0)
    zDoEvents() '// To process the pending messages, while checking the file.  

'   // Extract line from main text
    sTxt = MID$(sMainStr, nP, nCi - nP)          'extract line

    IF ASC(LTRIM$(sTxt)) = 39 THEN RETURN        'if whole line is uncommented, get next
  
'   // Blank out text within double quotes
    nQuotePos = INSTR(sTxt, $DQ)                 'see if there is any
    IF nQuotePos THEN
       nGetStrings = GetCheckButtonStatus(zGetMainItem(%IDC_CHECKBOX))
       DO                                        'loop while there is any left
          nQuotePos2 = INSTR(nQuotePos + 1, sTxt, $DQ) 'look for matching pair
          IF nQuotePos2 THEN
             IF WhatRun = 0 AND nGetStrings THEN 'if to extract string literals
                sDump = AnsiTRIM(MID$(sTxt, nQuotePos, nQuotePos2 - nQuotePos + 1))
                IF LEN(sDump) THEN
'                  // The word "PROGRAM" is missing from the exclude list in CheckInclude
                   IF CheckInclude(sDump) = 0 THEN
'                  // Replace Brje
'                  // ARRAY SCAN exclude(), FROM 1 TO LEN(sDump), COLLATE UCASE, = sDump, TO nAc
'                  // IF nAc = 0 THEN
                      IF gnStrCount MOD 20 = 0 THEN REDIM PRESERVE gsString(gnStrCount + 20)
                      gsString(gnStrCount) = LTRIM$(STR$(fNum)) + $TAB + USING$("####", I) + $TAB  + sDump
                      gnStrCount += 1
                   END IF
                END IF
             END IF
             MID$(sTxt, nQuotePos, nQuotePos2 - nQuotePos + 1) = SPACE$(nQuotePos2 - nQuotePos + 1)
             nQuotePos = INSTR(nQuotePos2 + 1, sTxt, $DQ)
             IF nQuotePos = 0 THEN EXIT DO
          ELSE
             EXIT DO
          END IF
       LOOP
    END IF
  
    sTxt = LTRIM$(EXTRACT$(sTxt, "'"))              'cut off ev. uncommented part and trim away leading spaces
    IF ASC(sTxt) = 82 AND LEFT$(sTxt, 4) = "REM " THEN RETURN 'if whole line in uncommented
    IF INSTR(UCASE$(sTxt), " REM ") THEN
       sTxt = LEFT$(sTxt, INSTR(UCASE$(sTxt), "REM ")) 'same here, if REM was used
    END IF
    IF INSTR(sTxt, " _") THEN
       sTxt = LEFT$(sTxt, INSTR(sTxt, " _") + 2)      'if line wraps to next, ignore the rest of it
    ELSEIF INSTR(sTxt, ",_ ") THEN
       sTxt = LEFT$(sTxt, INSTR(sTxt, ",_ ")) + " _"  'adjust to parser
    ELSEIF RIGHT$(sTxt, 2) = ",_" THEN
       sTxt = LEFT$(sTxt, LEN(sTxt) - 2) + " _"       'adjust to parser
    END IF
    IF ASC(sTxt, -1) = 32 THEN sTxt = RTRIM$(sTxt)    'trim off trailing spaces
  
    IF INSTR(sTxt, ":") THEN                       'colon inside paranthesis must be converted
       nPar = 0
       FOR bpLetter2 = STRPTR(sTxt) TO STRPTR(sTxt) + LEN(sTxt)
          SELECT CASE LONG @bpLetter2
             CASE 40 : nPar += 1                  'left paranthesis (
             CASE 41 : nPar -= 1                  'right paranthesis )
             CASE 58 : IF nPar > 0 THEN @bpLetter2 = 59 'if within paranthesis, convert colon to semicolon (whatever)
          END SELECT
       NEXT
    END IF
  
    IF LEN(sTxt) > 2 THEN                         'now, if line is enough long
       IF WhatRun = 0 THEN                        'and first run
          GOSUB ExtractSub                        'send it to sub/function check
          GOSUB ExtractGlobal                     'send it to global variable check
       ELSE
          IF LTRIM$(UCASE$(LEFT$(sTxt, 8))) = "DECLARE " OR ndUscored THEN
             ndUscored = (ASC(TRIM$(sTxt), -1) = 95)
             RETURN
          END IF
          GOSUB ChkVariables                      'second run, calculate globals
       END IF
    END IF

    RETURN

ExtractSub:
'   // Get subs and functions, plus get/check local variables (DIM, LOCAL, STATIC)
    IF nExFlag = 0 THEN
       sBuf = UcaseCrunchSpace(sTxt)             ' Version 1.06
  
       IF LEFT$(sBuf, 8) = "DECLARE " THEN       ' Declaration
          IF INSTR(sBuf, " LIB ") THEN RETURN    ' external routine - DLL, etc.
          sTxt = LTRIM$(MID$(sTxt, 9))
          sBuf = UCASE$(sTxt) : nFlag = 1
          IF LEN(RTRIM$(sTxt)) = 1 AND ASC(sTxt) = 95 THEN RETURN
       END IF
       
       IF LEFT$(sBuf, 9) = "FUNCTION " THEN                   ' Function start
          IF PARSE$(sBuf, $SPC, 2) = "AS" THEN RETURN          ' Version 1.06
          nExFlag = 2 : sFname = MID$(sTxt, 10): K = I
       ELSEIF LEFT$(sBuf, 4) = "SUB " THEN                    ' Sub start
          IF PARSE$(sBuf, $SPC, 2) = "AS" THEN RETURN          ' Version 1.06
          nExFlag = 1 : sFname = MID$(sTxt, 5) : K = I
       ELSEIF LEFT$(sBuf, 18) = "CALLBACK FUNCTION " THEN     ' Callback Function start
          nExFlag = 3 : sFname = MID$(sTxt, 19): K = I
       ELSEIF LEFT$(sBuf, 7) = "METHOD " THEN                 ' Method Function start
          IF PARSE$(sBuf, $SPC, 2) = "AS" THEN RETURN          ' Version 1.06
          nExFlag = 4 : sFname = MID$(sTxt, 8) : K = I
       END IF
  
       IF nExFlag THEN
          IF INSTR(sFname, " EXPORT") THEN nExported = 1
          nAc = INSTR(sFname, ANY " ('")
          IF nAc THEN sFname = TRIM$(LEFT$(sFname, nAc - 1), ANY " &%@!#$?")
       END IF
       IF LEN(sFname) = 1 AND ASC(sFname) = 95 THEN nUscored = 1
  
       IF nFlag AND nUscored = 0 THEN
          GOSUB AddDeclare : RETURN
       END IF
  
    ELSE
       IF nUscored THEN
          IF nFlag AND INSTR(sBuf, " LIB ") THEN RETURN             'external routine, DLL
          IF nUscored = 1 THEN 'look for name
             nAc = INSTR(LTRIM$(sTxt), ANY " (")
             IF nAc THEN
                sFname = TRIM$(LEFT$(sTxt, nAc - 1), ANY " &%@!#$?")
             ELSE
                sFname = TRIM$(sTxt, ANY " &%@!#$?")
             END IF
          ELSE
             IF INSTR(UCASE$(sTxt), "EXPORT") THEN nExported = 1
          END IF
          IF ASC(TRIM$(sTxt), -1) = 95 THEN
             nUscored = 2
          ELSE
             nUscored = 0
          END IF
       END IF
  
       IF nFlag AND nUscored = 0 THEN 'declaration
          GOSUB AddDeclare : RETURN
       END IF
  
       SELECT CASE LONG nExFlag
          CASE 1
             IF UCASE$(LEFT$(sTxt, 7)) = "END SUB" THEN
                nEndRout = nExFlag
             ELSE
                GOSUB ExtractLocals
                GOSUB ChkVariables
                RETURN
             END IF
          CASE 2, 3
             IF UCASE$(LEFT$(sTxt, 12)) = "END FUNCTION" THEN
                nEndRout = nExFlag
             ELSE
                GOSUB ExtractLocals
                GOSUB ChkVariables
                RETURN
             END IF
          CASE 4
             IF UCASE$(LEFT$(sTxt, 10)) = "END METHOD" THEN
                nEndRout = nExFlag
             ELSE
                GOSUB ExtractLocals
                GOSUB ChkVariables
                RETURN
             END IF
       END SELECT
  
       IF nEndRout THEN
          IF gnFunctions MOD 40 = 0 THEN REDIM PRESERVE gaFunctions(gnFunctions + 40)
          sFname = RTRIM$(sFname)
          gaFunctions(gnFunctions).zName    = sFname
          CALL UCASES(sFname)
          gaFunctions(gnFunctions).uName    = sFname
          gaFunctions(gnFunctions).iType    = nEndRout
          gaFunctions(gnFunctions).LineNum  = K
          gaFunctions(gnFunctions).SubEnd   = I
          gaFunctions(gnFunctions).FileNum  = fNum
          gaFunctions(gnFunctions).Exported = nExported
          gnFunctions += 1
          nExFlag = 0 : nEndRout = 0 : nExported = 0
  
          IF gnLocal THEN
             REDIM PRESERVE gaUnusedLocal(gnUnusedLocal + gnLocal)
             FOR nIv = 0 TO gnLocal - 1
                gaUnusedLocal(gnUnusedLocal).zName   = gaLocal(nIv).zName
                gaUnusedLocal(gnUnusedLocal).uName   = gaLocal(nIv).uName
                gaUnusedLocal(gnUnusedLocal).InFunct = gaLocal(nIv).InFunct
                gaUnusedLocal(gnUnusedLocal).iType   = gaLocal(nIv).iType
                gaUnusedLocal(gnUnusedLocal).LineNum = gaLocal(nIv).LineNum
                gaUnusedLocal(gnUnusedLocal).FileNum = gaLocal(nIv).FileNum
                gaUnusedLocal(gnUnusedLocal).IsUsed  = gaLocal(nIv).IsUsed
                gnUnusedLocal += 1
             NEXT
             gnLocal = 0 : REDIM gaLocal(0)
          END IF
       END IF
    END IF

    RETURN

AddDeclare:
    IF gnDeclCount MOD 40 = 0 THEN REDIM PRESERVE gaDeclareSubFunc(gnDeclCount + 40)
    sFname = RTRIM$(sFname)
    gaDeclareSubFunc(gnDeclCount).zName    = sFname
    CALL UCASES(sFname)
    gaDeclareSubFunc(gnDeclCount).uName    = sFname
    gaDeclareSubFunc(gnDeclCount).iType    = nExFlag
    gaDeclareSubFunc(gnDeclCount).LineNum  = K
    gaDeclareSubFunc(gnDeclCount).SubEnd   = I
    gaDeclareSubFunc(gnDeclCount).FileNum  = fNum
    gaDeclareSubFunc(gnDeclCount).Exported = nExported
    gnDeclCount += 1
    nExFlag = 0 : nEndRout = 0 : nExported = 0 : nFlag = 0
    RETURN

ExtractLocals:
'   // Get Locals
    IF INSTR(UCASE$(sTxt), "LOCAL ") OR INSTR(UCASE$(sTxt), "DIM ") OR _
       INSTR(UCASE$(sTxt), "STATIC ") OR nUscoredLocal THEN
       FOR nLocX = 1 TO PARSECOUNT(sTxt, ":")
          sWork = TRIM$(PARSE$(sTxt, ":", nLocX))
          IF UCASE$(LEFT$(sWork, 6)) = "LOCAL " OR _
             UCASE$(LEFT$(sWork, 4)) = "DIM " OR _
             UCASE$(LEFT$(sWork, 7)) = "STATIC " OR _
             nUscoredLocal THEN
  
              IF nUscoredLocal = 0 THEN
                 IF UCASE$(LEFT$(sWork, 6)) = "LOCAL " THEN
                    nIsGlobal = 0 : sWork = MID$(sWork, 7)
                 ELSEIF UCASE$(LEFT$(sWork, 4)) = "DIM " THEN
                    nIsGlobal = 1 : sWork = MID$(sWork, 5)    'start out by assuming global status
                 ELSEIF UCASE$(LEFT$(sWork, 7)) = "STATIC " THEN
                    nIsGlobal = 0 : sWork = MID$(sWork, 8)
                 END IF
              END IF
              FOR nLocY = 1 TO PARSECOUNT(sWork, ",")
                  sBuf = TRIM$(PARSE$(sWork, ",", nLocY))
  
                  IF nIsGlobal = 1 THEN 'check if DIM statement really was global
                     IF INSTR(UCASE$(sBuf), " GLOBAL") THEN    'this can only happen
                        nIsGlobal = 2                           'with "DIM xx AS GLOBAL.."
                     ELSEIF INSTR(UCASE$(sBuf), " LOCAL") OR _ 'local DIM..
                        INSTR(UCASE$(sBuf), " STATIC") THEN
                        nIsGlobal = 0
                     END IF
                  END IF
  
                  sBuf = EXTRACT$(sBuf, ANY " ()")  'Chop off AS LONG, etc, or if array - (
                  sBuf = RTRIM$(sBuf, ANY " &%@!#$?")
                  IF LEN(sBuf) = 1 AND ASC(sBuf) = 95 THEN ITERATE
                  sBuf += CHR$(0)
  
                  IF nIsGlobal < 2 THEN
                     ARRAY SCAN gaLocal(), FROM 1 TO LEN(sBuf), COLLATE UCASE, = sBuf, TO nLocPos
                     IF nLocPos = 0 THEN
                        IF gnLocal MOD 40 = 0 THEN REDIM PRESERVE gaLocal(gnLocal + 40)
                        gaLocal(gnLocal).zName   = sBuf
                        gaLocal(gnLocal).uName   = UCASE$(sBuf)
                        gaLocal(gnLocal).InFunct = sFname
                        gaLocal(gnLocal).FileNum = fNum
                        gaLocal(gnLocal).iType   = nIsGlobal
                        gaLocal(gnLocal).LineNum = I
                        gnLocal += 1
                     END IF
  
                  ELSE
                     ARRAY SCAN gaGlobal(), FROM 1 TO LEN(sBuf), COLLATE UCASE, = sBuf, TO nPos
                     IF nPos = 0 THEN 'if not already there, add it (GLOBAL+DIM/REDIM, DIM AS GLOBAL, etc.)
                        IF gnGlobal MOD 40 = 0 THEN REDIM PRESERVE gaGlobal(gnGlobal + 40)
                        gaGlobal(gnGlobal).zName = sBuf
                        gaGlobal(gnGlobal).uName = UCASE$(sBuf)
                        gaGlobal(gnGlobal).FileNum = fNum
                        gaGlobal(gnGlobal).LineNum = I
                        gnGlobal += 1
                     END IF
  
                  END IF
              NEXT
  
          END IF
       NEXT
       nUscoredLocal = (RIGHT$(RTRIM$(sTxt), 2)  = " _")
    END IF

    RETURN

ExtractGlobal:
'   // Get Globals
    IF INSTR(UCASE$(sTxt), "GLOBAL ") OR nUscoredGlobal THEN
       FOR nX = 1 TO PARSECOUNT(sTxt, ":")
          sWork = TRIM$(PARSE$(sTxt, ":", nX))
          nIsGlobal = (UCASE$(LEFT$(sWork, 7)) = "GLOBAL ")
          IF nIsGlobal = 0 THEN nIsGlobal = nUscoredGlobal
          IF UCASE$(LEFT$(sWork, 7)) = "GLOBAL " OR nUscoredGlobal THEN
              IF nUscoredGlobal = 0 THEN sWork = MID$(sWork, 8)
              FOR nY = 1 TO PARSECOUNT(sWork, ",")
                  sBuf = TRIM$(PARSE$(sWork, ",", nY))
                  sBuf = EXTRACT$(sBuf, ANY " ()")  'Chop off AS LONG etc.
                  sBuf = RTRIM$(sBuf, ANY " &%@!#$?")
                  IF LEN(sBuf) = 1 AND ASC(sBuf) = 95 THEN ITERATE
                  sBuf += CHR$(0)
  
                  IF gnGlobal THEN 'must check for ev. duplicate declarations
                     ARRAY SCAN gaGlobal(), FROM 1 TO LEN(sBuf), COLLATE UCASE, = sBuf, TO nDup
                     IF nDup THEN
                        IF gnDuplicateGlobal THEN
                           ARRAY SCAN gaDuplicateGlobal(), FROM 1 TO LEN(sBuf), COLLATE UCASE, = sBuf, TO nDup
                           IF nDup THEN
                              gaDuplicateGlobal(nDup - 1).IsUsed += 1
                              ITERATE FOR
                           END IF
                        END IF
                        REDIM PRESERVE gaDuplicateGlobal(gnDuplicateGlobal)
                        gaDuplicateGlobal(gnDuplicateGlobal).zName   = sBuf
                        gaDuplicateGlobal(gnDuplicateGlobal).uName   = UCASE$(sBuf)
                        gaDuplicateGlobal(gnDuplicateGlobal).FileNum = fNum
                        gaDuplicateGlobal(gnDuplicateGlobal).LineNum = I
                        gnDuplicateGlobal += 1
                        ITERATE FOR
                     END IF
                  END IF
  
                  IF gnGlobal MOD 40 = 0 THEN REDIM PRESERVE gaGlobal(gnGlobal + 40)
                  gaGlobal(gnGlobal).zName = sBuf
                  gaGlobal(gnGlobal).uName = UCASE$(sBuf)
                  gaGlobal(gnGlobal).FileNum = fNum
                  gaGlobal(gnGlobal).LineNum = I
                  gnGlobal += 1
              NEXT
  
          END IF
       NEXT
       IF nIsGlobal THEN nUscoredGlobal = ( RIGHT$(RTRIM$(sTxt), 2)  = " _" )
    END IF

    RETURN

ChkVariables:
'   // Check variables
    nWordFlag = 0 : nStrFlag = 0
    bpLetter2 = STRPTR(sTxt)
    FOR nDi = 1 TO LEN(sTxt)
       SELECT CASE LONG @bpLetter2
          CASE 97 TO 122, 65 TO 90, 48 TO 57, 95, 192 TO 214, 216 TO 246, 248 TO 255
             IF nWordFlag = 0 AND @bpLetter2 <> 95 THEN ' If valid char and no flag, word starts here (not with underscore)
                nWordFlag = 1 : nPos = nDi              ' set nWordFlag and store position
             END IF
  
          CASE ELSE                        ' We hit something else, like space, dot, etc.
             IF nWordFlag = 1 THEN         ' if flag, then a word is ready
                GOSUB ChkWord              ' check what we got
                nWordFlag = 0              ' and reset nWordFlag
             END IF
       END SELECT
       bpLetter2 += 1                      ' next char
    NEXT

    IF nWordFlag  = 1 THEN GOSUB ChkWord   ' In case there were letters all the way to the end..
    RETURN

ChkWord:
    nPos = nDi - nPos                              ' Calculate length
    sBuf = PEEK$(bpLetter2 - nPos, nPos) + CHR$(0) ' grab word
    CALL UCASES(sBuf)                              ' make it upper case for compare

    IF WhatRun = 0 THEN                         ' Check local variables
       ARRAY SCAN gaLocal(), FROM 1 TO LEN(sBuf), = sBuf, TO nPos
       IF nPos THEN gaLocal(nPos - 1).IsUsed += 1
  
    ELSE                                        ' Check Subs(Functions and Global vars
       ARRAY SCAN gaFunctions(), FROM 1 TO LEN(sBuf), = sBuf, TO nPos
       IF nPos THEN gaFunctions(nPos - 1).IsUsed += 1
  
       ARRAY SCAN gaGlobal(), FROM 1 TO LEN(sBuf), = sBuf, TO nPos
       IF nPos THEN gaGlobal(nPos - 1).IsUsed += 1
    END IF

    RETURN

END FUNCTION

'// Prepare and save a report of what we've found out!
SUB DoSaveResults
    LOCAL I, nBa, nRes, nDbl, ncTmp, nDone AS LONG
    LOCAL s1, s2, s3, s4, sWork, sFileName, sLine, sStr, sStr2 AS STRING

'   // Patrice bit array to skip un-used lines
    DIM bArray(gnTotLines \ 8) AS LONG

'   // First look through locals array. if iType = 1, it may be a GLOBAL DIM
    IF gnGlobal AND gnUnusedLocal THEN
       FOR I = gnUnusedLocal - 1 TO 0 STEP -1 '<- must run this backwards through array!
          IF gaUnusedLocal(I).iType = 1 THEN
             sStr = gaUnusedLocal(I).uName + CHR$(0)
             ARRAY SCAN gaGlobal(), FROM 1 TO LEN(sStr), = sStr, TO ncTmp
             IF ncTmp THEN               'if also GLOBAL, remove from local arrays
                ARRAY DELETE gaUnusedLocal(I)
                gnUnusedLocal -= 1
             END IF
          END IF
       NEXT
    END IF
    REDIM PRESERVE gaUnusedLocal(gnUnusedLocal)

'   // Now prepare report..
    gsDestFile = PARSE$(gsFileName, ANY ".", 1) + "_XREF.txt"
    CALL zUseWriteName(gsDestFile, 1)
    sWork = STRING$(80,"=")'"")
    sLine = STRING$(80,"-")

'   //------------------------------------------------------------------------------
    zWrite sWork
    s1 = UCASE$(gsFileName)
    s2 = DATE$
    s3 = TIME$
    zWrite BUILD$(" XRef report: ", s1, " + include files. Generated ", s2, ", ", s3)
    s1 = STR$(gnTotLines)
    s2 = LTRIM$(STR$(gdwTick / 1000)): IF ASC(S2) = 46 THEN S2 = "0" + RIGHT$(S2, 4)
    s3 = LTRIM$(str$(CLNG(gnTotLines / gdwTick * 60000)))
    zWrite BUILD$(s1, " lines scanned in ", s2, " seconds (", s3, " lines/minute)")
    zWrite sWork
'   //------------------------------------------------------------------------------
    IF UBOUND(gsFiles) > -1 THEN
       zWrite " MAIN + INCLUDE FILES"
       zWrite sLine
       gsIncludes(0) = "Main source file"
       FOR I = 0 TO UBOUND(gsFiles) - 1
           IF CheckInclude(PATHNAME$(NAMEX, gsFiles(I))) THEN 'these have been ignored
              sStr = " (not scanned)"
           ELSE
              sStr = ""
           END IF
           s1 = LEFT$(gsFiles(I) + sStr + SPACE$(58), 58)
           zWrite BUILD$($SPC, s1, "[", gsIncludes(I), "]")
       NEXT
    END IF
'   //------------------------------------------------------------------------------
    zWrite $SPC
    zWrite sWork
    zWrite " UN-USED FUNCTIONS  (exported, or in incl. files, may be used by other programs)"
    zWrite sLine
    IF gnFunctions THEN
       FOR I = 0 TO gnFunctions - 1
           IF gaFunctions(I).IsUsed = 1 AND gaFunctions(I).iType > 1 THEN
              IF CheckMain(gaFunctions(I).zName) = 0 THEN ' ignore these
                 sFileName = gsFiles(gaFunctions(I).FileNum)
                 sStr = " FUNCTION "
                 IF gaFunctions(I).iType = 3 THEN
                    sStr = " CALLBACK "
                 ELSEIF gaFunctions(I).iType = 4 THEN ' Version 1.06
                    sStr = " METHOD "
                 END IF
                 sStr2 = "": IF gaFunctions(I).Exported THEN sStr2 = " <EXPORT>"
                 s1 = LEFT$(sStr + gaFunctions(i).zName + sStr2 + SPACE$(52), 52)
                 s2 = PATHNAME$(NAMEX, sFileName)
                 s3 = STR$(gaFunctions(I).LineNum)
                 s4 = STR$(gaFunctions(I).SubEnd)
                 zWrite BUILD$(s1, "  [", s2, "] :", s3, " -", s4)

                 IF gaFunctions(I).Exported = 0 AND gaFunctions(I).iType <> 3 THEN
'                   // Flag the bit array for un-used functions
                    FOR nBa = gaFunctions(I).LineNum TO gaFunctions(I).SubEnd: BIT SET bArray(0), nBa: NEXT 
                 END IF

              END IF
          END IF
       NEXT
    END IF
'   //------------------------------------------------------------------------------
    zWrite $SPC
    zWrite sWork
    zWrite " UN-USED SUBS  (exported, or in incl. files, may be used by other programs)"
    zWrite sLine
    IF gnFunctions THEN
       FOR I = 0 TO gnFunctions - 1
           IF gaFunctions(I).IsUsed = 1 AND gaFunctions(I).iType = 1 THEN
              sFileName = gsFiles(gaFunctions(I).FileNum)
              sStr2 = "": IF gaFunctions(I).Exported THEN sStr2 = " <EXPORT>"
              s1 = LEFT$(" SUB " + gaFunctions(i).zName + sStr2 + SPACE$(50), 50)
              S2 = PATHNAME$(NAMEX, sFileName)
              s3 = STR$(gaFunctions(I).LineNum)
              s4 = STR$(gaFunctions(I).SubEnd)
              zWrite BUILD$(s1, "    [", s2, "] :", s3, " -", s4)

              IF gaFunctions(I).Exported = 0 THEN
'                // Flag the bit array for un-used sub
                 FOR nBa = gaFunctions(I).LineNum TO gaFunctions(I).SubEnd: BIT SET bArray(0), nBa: NEXT 
              END IF

           END IF
       NEXT
    END IF
'   //------------------------------------------------------------------------------
    IF gnDeclCount THEN
       zWrite $SPC
       zWrite sWork
       zWrite " DECLARED, BUT NON-EXISTING SUB/FUNCTION(S)"
       zWrite sLine
    
       FOR I = 0 TO gnDeclCount - 1
          IF gnFunctions > 0 THEN
             sStr = gaDeclareSubFunc(I).uName + CHR$(0)
             ARRAY SCAN gaFunctions(), FROM 1 TO LEN(sStr), = sStr, TO nDbl
          END IF
          IF nDbl = 0 THEN
             sFileName = gsFiles(gaDeclareSubFunc(I).FileNum)
             IF gaDeclareSubFunc(I).iType = 1 THEN
                sStr = " SUB "
             ELSEIF gaDeclareSubFunc(I).iType = 2 THEN
                sStr = " FUNCTION "
             ELSEIF gaDeclareSubFunc(I).iType = 3 THEN
                sStr = " CALLBACK "
             ELSEIF gaDeclareSubFunc(I).iType = 4 THEN ' Version 1.06
                sStr = " METHOD "
             END IF
             sStr2 = "": IF gaDeclareSubFunc(I).Exported  THEN sStr2 = " <EXPORT>"
             s1 = LEFT$(sStr + gaDeclareSubFunc(I).zName + sStr2 + SPACE$(50), 50)
             s2 = PATHNAME$(NAMEX, sFileName)
             s3 = STR$(gaDeclareSubFunc(I).LineNum)
             zWrite BUILD$(s1, "    [", s2, "] :", s3)
          END IF
       NEXT
    END IF
'   //------------------------------------------------------------------------------
    IF gnFunctions THEN
       zWrite $SPC
       zWrite sWork
       zWrite " EXISTING, BUT NON-DECLARED SUB/FUNCTION(S)"
       zWrite sLine
  
       FOR I = 0 TO gnFunctions - 1
          IF gnDeclCount THEN
             sStr = gaFunctions(I).uName + CHR$(0)
             ARRAY SCAN gaDeclareSubFunc(), FROM 1 TO LEN(sStr), = sStr, TO nDbl
          END IF
          IF nDbl = 0 THEN
             IF CheckMain(gaFunctions(I).zName) = 0 THEN ' ignore these
                sFileName = gsFiles(gaFunctions(I).FileNum)
                IF gaFunctions(I).iType = 1 THEN
                   sStr = " SUB "
                ELSEIF gaFunctions(I).iType = 2 THEN
                   sStr = " FUNCTION "
                ELSEIF gaFunctions(I).iType = 3 THEN
                   sStr = " CALLBACK "
                ELSEIF gaFunctions(I).iType = 4 THEN
                   sStr = " METHOD "
                END IF
                sStr2 = "": IF gaFunctions(I).Exported  THEN sStr2 = " <EXPORT>"
                s1 = USING$("####", gaFunctions(I).IsUsed - 1)
                s2 = LEFT$(sStr + gaFunctions(I).zName + sStr2 + SPACE$(45), 45)
                s3 = PATHNAME$(NAMEX, sFileName)
                s4 = STR$(gaFunctions(I).LineNum)
                zWrite BUILD$(s1, s2, "    [", s3, "] :", s4)
             END IF
          END IF
       NEXT
    END IF
'   //------------------------------------------------------------------------------
    zWrite $SPC
    zWrite sWork
    zWrite " UN-USED GLOBAL VARIABLES"
    zWrite sLine
    IF gnGlobal THEN
       FOR I = 0 TO gnGlobal - 1
          IF gaGlobal(I).IsUsed = 1 THEN
             sFileName = gsFiles(gaGlobal(I).FileNum)
             s1 = LEFT$(gaGlobal(i).zName + SPACE$(47), 47)
             s2 = PATHNAME$(NAMEX, sFileName)
             s3 = STR$(gaGlobal(I).LineNum)
             zWrite BUILD$($SPC, s1, "    [", s2, "] :", s3)
          END IF
       NEXT
    END IF
'   //------------------------------------------------------------------------------
    zWrite $SPC
    zWrite sWork
    zWrite " UN-USED LOCAL VARIABLES"
    zWrite sLine
    IF gnUnusedLocal THEN
       FOR I = 0 TO gnUnusedLocal - 1
          IF gaUnusedLocal(I).IsUsed = 1 THEN
             sFileName = gsFiles(gaUnusedLocal(I).FileNum)
             s1 = LEFT$(gaUnusedLocal(i).zName + SPACE$(47), 47)
             s2 = PATHNAME$(NAMEX, sFileName)
             s3 = STR$(gaUnusedLocal(I).LineNum)
             zWrite BUILD$($SPC, s1, "    [", s2, "] :", s3)
          END IF
       NEXT
    END IF
'   //------------------------------------------------------------------------------
    IF gnGlobal AND gnUnusedLocal THEN
       FOR I = 0 TO gnGlobal - 1
          sStr = gaGlobal(I).uName + CHR$(0)
          ARRAY SCAN gaUnusedLocal(), FROM 1 TO LEN(sStr), = sStr, TO ncTmp
          IF ncTmp THEN EXIT FOR
       NEXT
       IF ncTmp THEN
          zWrite $SPC
          zWrite sWork
          zWrite " GLOBAL/LOCAL MIX - WARNING!"
          zWrite " Following global variable name(s) exist in both global and local"
          zWrite " form. While the compiler allows this, special care must be taken"
          zWrite " to avoid hard-to-find errors. Please check them out carefully."
          zWrite sLine
    
          FOR I = 0 TO gnGlobal - 1
             sStr = gaGlobal(I).uName + CHR$(0)
             ARRAY SCAN gaUnusedLocal(), FROM 1 TO LEN(sStr), = sStr, TO nRes
             IF nRes THEN
                ncTmp = 0 : nDbl = 0
                sFileName = gsFiles(gaGlobal(I).FileNum)
                s1 = LEFT$(gaGlobal(I).zName + SPACE$(47), 47)
                s2 = PATHNAME$(NAMEX, sFileName)
                s3 = STR$(gaGlobal(I).LineNum)
                zWrite BUILD$($SPC, s1, "    [", s2, "] :", s3)
                DO
                   ncTmp = ncTmp + nRes
                   sFileName = gsFiles(gaUnusedLocal(ncTmp - 1).FileNum)
                   s1 = PATHNAME$(NAMEX, sFileName)
                   s2 = STR$(gaUnusedLocal(ncTmp - 1).LineNum)
                   zWrite BUILD$("   local in ", s1, " :", s2)
                   ARRAY SCAN gaUnusedLocal(ncTmp), FROM 1 TO LEN(sStr), = sStr, TO nRes
                LOOP WHILE nRes
             END IF
          NEXT
       END IF
    END IF
'   //------------------------------------------------------------------------------
    IF gnDuplicateGlobal THEN
       zWrite $SPC
       zWrite sWork
       zWrite " DUPLICATE GLOBAL NAMES - WARNING!"
       zWrite " Following global name(s) exist as both array and varable."
       zWrite " While the compiler allows this, special care must be taken"
       zWrite " avoid hard-to-find errors. Please check them out carefully."
       zWrite sLine
       FOR I = 0 TO gnDuplicateGlobal - 1
          sFileName = gsFiles(gaDuplicateGlobal(I).FileNum)
          s1 = LEFT$(gaDuplicateGlobal(I).zName + SPACE$(47), 47)
          s2 = PATHNAME$(NAMEX, sFileName)
          s3 = STR$(gaDuplicateGlobal(I).LineNum)
          zWrite BUILD$($SPC, s1, "    [", s2, "] :", s3)
       NEXT
    END IF
'   //------------------------------------------------------------------------------
    zWrite $SPC
    zWrite sWork
    zWrite " TOTAL REFERENCE COUNT - (Count, Name, [declared in File] : at Line number)"
    zWrite " Lists how many times the following has been called/used (zero = un-used)"
    zWrite sLine
'   //------------------------------------------------------------------------------
    IF gnFunctions > 0 THEN
       zWrite " FUNCTIONS:"
       FOR I = 0 TO gnFunctions - 1
          IF gaFunctions(I).iType > 1 THEN
             IF CheckMain(gaFunctions(I).zName) = 0 THEN ' ignore these
                sFileName = gsFiles(gaFunctions(I).FileNum)
                sStr = ""
                IF gaFunctions(I).Exported THEN sStr = " <EXPORT>"
                s1 = USING$("####", gaFunctions(I).IsUsed - 1)
                s2 = LEFT$(gaFunctions(I).zName + sStr + SPACE$(43), 43)
                s3 = PATHNAME$(NAMEX, sFileName)
                s4 = STR$(gaFunctions(I).LineNum)
                zWrite BUILD$(s1, "  ", s2, "    [", s3, "] :", s4)
             END IF
          END IF
       NEXT
    END IF
'   //------------------------------------------------------------------------------
    zWrite $SPC
    IF gnFunctions > 0 THEN
       zWrite " SUBS:"
       FOR I = 0 TO gnFunctions - 1
          IF gaFunctions(I).iType = 1 THEN
             sFileName = gsFiles(gaFunctions(I).FileNum)
             sStr = ""
             IF gaFunctions(I).Exported THEN sStr = " <EXPORT>"
             s1 = USING$("####", gaFunctions(I).IsUsed - 1)
             s2 = LEFT$(gaFunctions(I).zName + sStr + SPACE$(43), 43)
             s3 = PATHNAME$(NAMEX, sFileName)
             s4 = STR$(gaFunctions(I).LineNum)
             zWrite BUILD$(s1, "  ", s2, "    [", s3, "] :", s4)
          END IF
        NEXT
    END IF
'   //------------------------------------------------------------------------------
    zWrite $SPC
    IF gnGlobal > 0 THEN
       zWrite " GLOBAL VARIABLES:"
       FOR I = 0 TO gnGlobal - 1
          sFileName = gsFiles(gaGlobal(I).FileNum)
          s1 = USING$("####", gaGlobal(I).IsUsed - 1)
          s2 = LEFT$(gaGlobal(I).zName + SPACE$(43), 43)
          s3 = PATHNAME$(NAMEX, sFileName)
          s4 = STR$(gaGlobal(I).LineNum)
          zWrite BUILD$(s1, "  ", s2, "    [", s3, "] :", s4)
       NEXT
    END IF

'   //------------------------------------------------------------------------------
'   // STRING LITERALS
    IF gnStrCount THEN
       zWrite $SPC
       zWrite sWork
       zWrite " STRING LITERALS"
       sFileName = ""
    
       FOR I = 0 TO gnStrCount - 1
          sStr = gsFiles(VAL(PARSE$(gsString(I), $TAB, 1)))
          sStr =PATHNAME$(NAMEX, sStr)
          IF sStr <> sFileName THEN
             sFileName = sStr
             zWrite $SPC
             IF I THEN zWrite sLine
             zWrite BUILD$(" Line  Text     [", sFileName, "]")
             zWrite sLine
          END IF
          s1 = PARSE$(gsString(I), $TAB, 2)
          s2 = PARSE$(gsString(I), $TAB, 3)
          zWrite BUILD$(s1, "  ", s2)
       NEXT
    END IF
'   //------------------------------------------------------------------------------

    zWrite "" ' Close file
  
    CALL apiSleep(20)
'   // Launch log file
    CALL ShellExecute(0, "open", gsFilePath + gsDestFile, BYVAL 0, BYVAL 0, %SW_SHOWNORMAL)

'   // Patrice build the cleaned version
    IF GetCheckButtonStatus(zGetMainItem(%IDC_CHKBOXCLEAN)) THEN
       CALL BuildCleanVersion(bArray())
    END IF

END SUB

'// GetDroppedFile - Function Loads File/Folder names into the global arrays
FUNCTION GetDroppedFile( BYVAL hfInfo AS LONG, Fi() AS STRING) AS LONG
    LOCAL nCount AS LONG, ln AS LONG, tmp AS STRING, fName AS ASCIIZ * %MAX_PATH
    
    nCount = DragQueryFile(hfInfo, &HFFFFFFFF&, BYVAL %NULL, 0)' get number of dropped files
    
    IF nCount THEN                                             ' If we got something
       ln = DragQueryFile(hfInfo, 0, fName, %MAX_PATH)         ' put FileName into fString And get len
       IF ln THEN
          tmp = TRIM$(LEFT$(fName, ln))
          IF LEN(tmp) AND (GETATTR(tmp) AND 16) = 0 THEN       ' make sure it's a file, not a folder
             Fi(0) = tmp
             FUNCTION = 1
          END IF
       END IF
    END IF

    CALL DragFinish(hfInfo)
END FUNCTION

'// Check to see if the file has a #COMPILE metastatement
FUNCTION IsFileMain(BYVAL sFileName AS STRING) AS LONG
    LOCAL nErrCode, nRemaining, hFile, nDone, nAsc AS LONG
    LOCAL sBuffer AS STRING

    nErrCode = zFOpen((sFileName), 0, 2, hFile)' open file
    IF nErrCode THEN                           ' if it failed
       FUNCTION = -2 : EXIT FUNCTION           ' return -2 to indicate failure and exit
    END IF

    nRemaining = zFLof(hFile)                  ' Bytes count to be read
    CALL zFClose(hFile)                        ' close it
    IF nRemaining = 0 THEN                     ' if zero length file
       FUNCTION = -3 : EXIT FUNCTION           ' return -3 to indicate empty file and exit
    END IF

    nDone = 0
    DO WHILE nDone = 0
       sBuffer = LTRIM$(zBufin((sFileName), nDone), ANY CHR$(32,9))
       IF LEN(sBuffer) > 8 THEN
          sBuffer = UCASE$(sBuffer)            'adjust sBuffer for compare
          nAsc = ASC(sBuffer): IF nAsc = 36 THEN nAsc = 35
          IF nAsc = 35 AND MID$(sBuffer, 2, 8) = "COMPILE " THEN
              FUNCTION = -1
              EXIT DO
          END IF

          IF LEFT$(sBuffer, 9) = "FUNCTION " OR _  'jump out once we hit a Sub or Function
             LEFT$(sBuffer, 4) = "SUB " OR _
                LEFT$(sBuffer, 9) = "CALLBACK " OR _
                   LEFT$(sBuffer, 7) = "STATIC " OR _
                      LEFT$(sBuffer, 7) = "METHOD " THEN

             FUNCTION = nRemaining 'return length
             EXIT DO
          END IF
       END IF
    LOOP
    IF nDone = 0 THEN CALL zBufin("", nDone)

END FUNCTION

'// UCASES routine, alters given string directly - very fast..
SUB UCASES(a$)
  IF LEN(a$) = 0 THEN EXIT SUB
  #REGISTER NONE

  ! mov eax, a$            ; eax = pointer to string handle
  ! mov eax, [eax]         ; eax = pointer to string data
  ! mov ecx, [eax-4]       ; move length of string into ecx

  BgnUpLoop:
     ! mov dl, [eax]       ; move current char into dl
     ! cmp dl, 97          ; compare against value 97 (a)
     ! jb NoDnChar         ; if dl < 97 it is in 0-97 range, do nothing - get next
     ! cmp dl, 123         ; compare against value 123
     ! jb MakeUpper        ; if dl < 123 it is in 97-122 range, make Upper case and get next
     ! cmp dl, 224         ; compare against value 224 () - extended ANSI characters
     ! jb NoDnChar         ; if dl < 224 it is in 123-223 range, do nothing - get next
     ! cmp dl, 247         ; compare against value 247 ()
     ! jb MakeUpper        ; if dl < 247 it is in 224-246 range, make Upper case and get next
     ! je NoDnChar         ; else dl = 247 (), do nothing - get next
     ! cmp dl, 255         ; compare against value 255 ()
     ! jb MakeUpper        ; if dl < 255 it is in 248-254 range, make Upper case and get next
     ! jmp NoDnChar        ; else dl = 255, do nothing - get next

  MakeUpper:
     ! sub dl, 32          ; make uppercase subtracting 32 from dl's value
     ! mov [eax], dl       ; write changed char back into a$

  NoDnChar:
     ! inc eax             ; get next character
     ! dec ecx             ; decrease ecx (length) counter
     ! jnz bgnUpLoop       ; iterate if not zero
END SUB

'// #STACK 8388608 ' (8Mb)

'// Patrice - Build the combined file (Main + Include files)
SUB BuildSingleFile(BYVAL sFileName AS STRING)
    LOCAL nUseVersion, nUseMacro, nDetectMacro, nDoIt, nIncludeFlag, nCar, nP, nLen, nAsc, hFile AS LONG
    LOCAL sInclude, sItem, sConstant, sTmp, sBuf, sDetect, sLineStr AS STRING

    STATIC nCount, nRemove AS LONG

'   // Version 1.05 They must be included only once!
    sLineStr = UCASE$(PATHNAME$(NAMEX, sFileName))
    ARRAY SCAN gsIncludes(), COLLATE UCASE, = sLineStr, TO nP
    IF nP THEN EXIT SUB

    nP = MAX&(UBOUND(gsIncludes) + 1, 0)
    REDIM PRESERVE gsIncludes(nP)
    gsIncludes(nP) = sLineStr
'   // Version 1.05 End

    IF zExist(sFileName) = 0 THEN sFileName = zFindFile(sFileName)
    IF zExist(sFileName) = 0 THEN EXIT SUB

    IF zFOpen((sFileName), 0, 2, hFile) = 0 THEN
       nLen = zFLof(hFile)
       sTmp = SPACE$(nLen)
       CALL zFGet(hFile, sTmp)
       CALL zFClose(hFile)
    END IF

    IF sFileName = gsFiles(0) THEN nIncludeFlag = 0 ELSE nIncludeFlag = -1
'   IF nIncludeFlag THEN zBuild($CRLF + "'>>> MERGE START: " + sFileName + $CRLF + $CRLF, 1)

'   // Let the user know the name of the file we are processing
    CALL zSetCTLText(zGetMainItem(%IDC_HELPER), "Processing" + $cr + sLineStr)

    nUseVersion = 0
    nP = 1
'    LOCAL nTrace AS LONG
    FOR nCar = 1 TO nLen

        IF ASC(sTmp, nCar) = 13 THEN

           CALL SendMessage(zGetMainItem(%IDC_PROGRESS), 1026, (nCar * 100) \ nLen, 0)
           zDoEvents() '// To process the pending messages, while checking the file.  

           IF nCar < nLen AND ASC(sTmp, nCar + 1) = 10 THEN nCar += 1
           sBuf = MID$(sTmp, nP, nCar + 1 - nP)
           nDoIt = -1

           IF nIncludeFlag THEN
              IF ASC(LTRIM$(sBuf)) = 39 THEN sBuf = ""
              IF LEN(sBuf) = 0 OR ASC(sBuf) = 13 THEN nDoIt = 0
              IF nDoIt THEN 
'                 nTrace = 0
                 sDetect = EXTRACT$(sBuf, CHR$(13)): REPLACE ANY "()" WITH "  " IN sDetect 
                 sDetect = UcaseCrunchSpace(sDetect)

                 IF ASC(sDetect) = 37 THEN '// %
                    IF ASC(PARSE$(sDetect, $SPC, 2)) = 61 THEN '// =
                       sItem = EXTRACT$(sDetect, $SPC)
                       IF ListExist(%ID_CONSTANT, sItem) = 0 THEN
                          sConstant = PARSE$(sDetect, $SPC, 3)
                          IF ASC(sConstant) <> 37 THEN
                             SetStaticValue(%ID_STATIC, sItem, VAL(sConstant))
                          END IF
                       END IF
                    END IF
                 END IF

                 IF nRemove THEN nDoIt = 0
                 IF gnFlagDEF THEN nDoIt = 0
                 IF ASC(sDetect) = 35 THEN ' #

                    IF LEFT$(sDetect, 4) = "#IF " THEN
                       gnIndent = gnCountIF * 3
'                       nTrace = 1

                       gnCountIF += 1
                       IF LEFT$(sDetect, 13) = "#IF NOT %DEF " THEN
                          sItem = PARSE$(sDetect, $SPC, 4)
                          IF ListExist(%ID_STATIC, sItem) = 0 THEN
                             SetStaticValue(%ID_STATIC, sItem, 0)
                          ELSE
                             IF gnCountIF = 1 THEN nRemove += 1
                          END IF
                       ELSEIF LEFT$(sDetect, 9) = "#IF %DEF " THEN
                          IF gnFlagDEF = 0 THEN
                             sItem = PARSE$(sDetect, $SPC, 3)
                             IF sItem = "%USEMACROS" THEN
                                IF nDetectMacro = 0 THEN
                                   IF nUseMacro THEN nDetectMacro = 2 ELSE nDetectMacro = 3
                                   nDoIt = 0
                                END IF                         
                             ELSEIF ListExist(%ID_CONSTANT, sItem) = 0 THEN
                                gnFlagDEF = gnIndent + 1: nDoIt = 0
'zTrace(str$(gnIndent)+"<"+string$(80, "1"))
                             END IF
                          END IF
'                       ELSEIF LEFT$(sDetect, 12) = "#IF %WINVER " THEN
'                          IF gnCountIF THEN 
'                             gnCountIF += 1': nDoIt = 0
'                          ELSE
'                             nUseVersion = 1
'                          END IF

                       END IF

                    ELSEIF sDetect = "#ELSE" THEN
                       gnIndent = gnCountIF * 3
'                       nTrace = 1

                       'IF nUseVersion THEN
                       'ELSE
                       IF nDetectMacro > 0 AND nDetectMacro < 4 THEN 
                          nDetectMacro = nDetectMacro * 2: nDoIt = 0
                       END IF
                    ELSEIF sDetect = "#ENDIF" THEN
                       IF gnCountIF > 0 THEN
                          gnCountIF -= 1
                          IF gnCountIF = 0 THEN
                             IF nRemove > 0 THEN nRemove -= 1 
                          END IF
                       END IF'ELSE
'                          IF nUseVersion THEN
'                             nUseVersion = 0
'                          ELSE
                       IF nDetectMacro THEN 
                          nDetectMacro = 0: nDoIt = 0
                       END IF
                       
                       gnIndent = gnCountIF * 3
'                       nTrace = 1

                       IF gnFlagDEF THEN
                          IF gnFlagDEF = gnIndent + 1 THEN
                             gnFlagDEF = 0
'zTrace(str$(gnIndent)+"<"+string$(80, "2"))
                          END IF
                       END IF

                    END IF
                 END IF
              END IF
           END IF
           nP = nCar + 1
           IF nDetectMacro = 3 OR nDetectMacro = 4 THEN nDoIt = 0
           IF nDoIt THEN
              sLineStr = UCASE$(LTRIM$(sBuf))
              nASC = ASC(sLineStr): IF nASC = 36 THEN nASC = 35
              IF nASC = 35 AND MID$(sLineStr, 2, 8) = "INCLUDE " THEN
                 sInclude = PARSE$(sLineStr, CHR$(34), 2)
                 IF CheckInclude(sInclude) = 0 THEN
                    CALL BuildSingleFile(sInclude)  ' Start recursive concatenation
                 ELSE
                    zBuild(sBuf, 1)
gnLineCount += 1
'if nTrace THEN zTrace(RIGHT$(space$(5)+str$(gnLineCount), 6) + ">" + space$(gnIndent)+sBuf)
                 END IF
              ELSE
                 zBuild(sBuf, 1)
gnLineCount += 1
'if nTrace THEN zTrace(RIGHT$(space$(5)+str$(gnLineCount), 6) + "|" + space$(gnIndent)+sBuf)
              END IF
           END IF
        END IF
    NEXT
    IF nP < nLen THEN 
       zBuild(MID$(sTmp, nP, nLen - nP + 1), 1)
       gnLineCount += 1
'if nTrace THEN zTrace(RIGHT$(space$(5)+str$(gnLineCount), 6) + "" + space$(gnIndent)+sBuf)
    END IF
'IF nIncludeFlag THEN 
'   zBuild($CRLF + "'>>> MERGE END: " + sFileName + $CRLF + $CRLF, 1)
'gnLineCount += 3
'if nTrace THEN zTrace(RIGHT$(space$(5)+str$(gnLineCount), 6) + "|" + space$(gnIndent)+sBuf)
'END IF

'   // Reset ProgressBar to null
    CALL SendMessage(zGetMainItem(%IDC_PROGRESS), 1026, 0, 0)

END SUB

SUB BuildCleanVersion(bArray() AS LONG)
    LOCAL zFileName AS ASCIIZ * %MAX_PATH, sFileOutput, sBuf AS STRING
    LOCAL nDone, nLine, nRemoved AS LONG
    nDone = 0: nLine = 0: nRemoved = 0
    zFileName = gsFilePath + gsFileName
    sFileOutput = gsFilePath + "$" + MID$(gsFileName, 2, LEN(gsFileName) - 1)
    CALL zUseBuildName(sFileOutput, 1)
    DO WHILE nDone = 0
       sBuf = zBufin(zFileName, nDone)
       IF nDone = 0 THEN
          nLine += 1
          IF BIT(bArray(0), nLine) = 0 THEN
             'IF ASC(LTRIM$(sBuf)) = 39 THEN sBuf = ""   '// 08-26-2009
             'IF LEN(sBuf) THEN zBuild(sBuf + $CRLF, 1)  '// 08-26-2009
             zBuild(sBuf + $CRLF, 1)
          ELSE
             nRemoved +=1 ' Removed line counter
          END IF
       END IF
    LOOP
    zBuild("", 0)

    IF nRemoved THEN
       apiSleep(500)
       sBuf = "Cleaned file saved as:" + $CR + sFileOutput + $CR + $CR + _
              LTRIM$(STR$(nRemoved)) + " Un-used lines have been removed" + $cr + "from source code."  
       MSGBOX sBuf,, EXE.Name$ + $SPC + $VERSION
    END IF

END SUB
